[PATCH 4/6] lib/parse-sexp: support actual date queries.

Subject: [PATCH 4/6] lib/parse-sexp: support actual date queries.

Date: Mon, 3 Jan 2022 22:55:27 -0400

To: notmuch@notmuchmail.org

Cc:

From: David Bremner


The default argument processing overlaps somewhat with what is already
done in _notmuch_date_strings_to_query, but we can give more specific
error messages for the s-expression context.

The extra generality of _sexp_parse_range will be useful when we
implement additional range prefixes (at least 'lastmod' is needed).
---
 lib/parse-sexp.cc         | 41 +++++++++++++++++++++++++++++++++------
 test/T081-sexpr-search.sh |  5 -----
 2 files changed, 35 insertions(+), 11 deletions(-)

diff --git a/test/T081-sexpr-search.sh b/test/T081-sexpr-search.sh
index a6e2ee82..ea55c2b2 100755
--- a/test/T081-sexpr-search.sh
+++ b/test/T081-sexpr-search.sh
@@ -774,13 +774,11 @@ notmuch search --query=sexp  '(and (date) (from keithp))'| notmuch_search_saniti
 test_expect_equal_file EXPECTED OUTPUT
 
 test_begin_subtest "date query, one argument"
-test_subtest_known_broken
 notmuch search date:2009-11-18 and from:keithp | notmuch_search_sanitize > EXPECTED
 notmuch search --query=sexp  '(and (date 2009-11-18) (from keithp))' | notmuch_search_sanitize > OUTPUT
 test_expect_equal_file EXPECTED OUTPUT
 
 test_begin_subtest "date query, two arguments"
-test_subtest_known_broken
 notmuch search date:2009-11-17..2009-11-18 and from:keithp | notmuch_search_sanitize > EXPECTED
 notmuch search --query=sexp  '(and (date 2009-11-17 2009-11-18) (from keithp))' | notmuch_search_sanitize > OUTPUT
 test_expect_equal_file EXPECTED OUTPUT
@@ -802,7 +800,6 @@ EOF
 test_expect_equal_file EXPECTED OUTPUT
 
 test_begin_subtest "date query, illegal nesting 3"
-test_subtest_known_broken
 notmuch search --query=sexp '(date (to))' > OUTPUT 2>&1
 cat <<EOF > EXPECTED
 notmuch search: Syntax error in query
@@ -811,7 +808,6 @@ EOF
 test_expect_equal_file EXPECTED OUTPUT
 
 test_begin_subtest "date query, illegal nesting 4"
-test_subtest_known_broken
 notmuch search --query=sexp '(date today (to))' > OUTPUT 2>&1
 cat <<EOF > EXPECTED
 notmuch search: Syntax error in query
@@ -820,7 +816,6 @@ EOF
 test_expect_equal_file EXPECTED OUTPUT
 
 test_begin_subtest "date query, too many arguments"
-test_subtest_known_broken
 notmuch search --query=sexp '(date yesterday and tommorow)' > OUTPUT 2>&1
 cat <<EOF > EXPECTED
 notmuch search: Syntax error in query
diff --git a/lib/parse-sexp.cc b/lib/parse-sexp.cc
index f36d18a6..fa88071e 100644
--- a/lib/parse-sexp.cc
+++ b/lib/parse-sexp.cc
@@ -450,15 +450,45 @@ _sexp_expand_param (notmuch_database_t *notmuch, const _sexp_prefix_t *parent,
 }
 
 static notmuch_status_t
-_sexp_parse_date (notmuch_database_t *notmuch, const sexp_t *sx, Xapian::Query &output)
+_sexp_parse_range (notmuch_database_t *notmuch,  const _sexp_prefix_t *prefix,
+		   const sexp_t *sx, Xapian::Query &output)
 {
-    /* empty date matches everything */
+    const char *from, *to;
+    std::string msg; /* ignored */
+
+    /* empty range matches everything */
     if (! sx) {
 	output = Xapian::Query::MatchAll;
 	return NOTMUCH_STATUS_SUCCESS;
     }
 
-    _notmuch_database_log (notmuch, "unimplemented date query\n");
+    if (sx->ty == SEXP_LIST) {
+	_notmuch_database_log (notmuch, "expected atom as first argument of '%s'\n", prefix->name);
+	return NOTMUCH_STATUS_BAD_QUERY_SYNTAX;
+    }
+
+    from = sx->val;
+    to = from;
+
+    if (sx->next) {
+	if (sx->next->ty == SEXP_LIST) {
+	    _notmuch_database_log (notmuch, "expected atom as second argument of '%s'\n",
+				   prefix->name);
+	    return NOTMUCH_STATUS_BAD_QUERY_SYNTAX;
+	}
+
+	if (sx->next->next) {
+	    _notmuch_database_log (notmuch, "'%s' expects maximum of two arguments\n", prefix->name);
+	    return NOTMUCH_STATUS_BAD_QUERY_SYNTAX;
+	}
+
+	to = sx->next->val;
+    }
+
+    if (strcmp (prefix->name, "date") == 0)
+	return _notmuch_date_strings_to_query (NOTMUCH_VALUE_TIMESTAMP, from, to, output, msg);
+
+    _notmuch_database_log (notmuch, "unimplimented range prefix: '%s'\n", prefix->name);
     return NOTMUCH_STATUS_BAD_QUERY_SYNTAX;
 }
 
@@ -557,9 +587,8 @@ _sexp_to_xapian_query (notmuch_database_t *notmuch, const _sexp_prefix_t *parent
 		return NOTMUCH_STATUS_BAD_QUERY_SYNTAX;
 	    }
 
-	    if (strcmp (prefix->name, "date") == 0) {
-		return _sexp_parse_date (notmuch, sx->list->next, output);
-	    }
+	    if (prefix->flags & SEXP_FLAG_RANGE)
+		return _sexp_parse_range (notmuch, prefix, sx->list->next, output);
 
 	    if (strcmp (prefix->name, "infix") == 0) {
 		return _sexp_parse_infix (notmuch, sx->list->next, output);
-- 
2.34.1

_______________________________________________
notmuch mailing list -- notmuch@notmuchmail.org
To unsubscribe send an email to notmuch-leave@notmuchmail.org

Thread: